/**
 * @license Copyright (c) 2003-2019, CKSource - Frederico Knabben. All rights reserved.
 * For licensing, see LICENSE.md.
 */

/**
 * @module highlight/highlightediting
 */

import Plugin from '@ckeditor/ckeditor5-core/src/plugin';

import HighlightCommand from './highlightcommand';

/**
 * The highlight editing feature. It introduces the {@link module:highlight/highlightcommand~HighlightCommand command} and the `highlight`
 * attribute in the {@link module:engine/model/model~Model model} which renders in the {@link module:engine/view/view view}
 * as a `<mark>` element with a `class` attribute (`<mark class="marker-green">...</mark>`) depending
 * on the {@link module:highlight/highlight~HighlightConfig configuration}.
 *
 * @extends module:core/plugin~Plugin
 */
export default class HighlightEditing extends Plugin {
	/**
	 * @inheritDoc
	 */
	constructor( editor ) {
		super( editor );

		editor.config.define( 'highlight', {
			options: [
         {model: 'black1', class: 'ck5-pen-black1', title: 'black1', color: 'var(--ck-highlight-pen-black1)', type: 'pen'},			
         {model: 'white1', class: 'ck5-pen-white1', title: 'White1', color: 'var(--ck-highlight-pen-white1)', type: 'pen'},     
         {model: 'beige1', class: 'ck5-pen-beige1', title: 'beige1', color: 'var(--ck-highlight-pen-beige1)', type: 'pen'},
         {model: 'blue1', class: 'ck5-pen-blue1', title: 'blue1', color: 'var(--ck-highlight-pen-blue1)', type: 'pen'},
         {model: 'red1', class: 'ck5-pen-red1', title: 'red1', color: 'var(--ck-highlight-pen-red1)', type: 'pen'},
         {model: 'green1', class: 'ck5-pen-green1', title: 'green1', color: 'var(--ck-highlight-pen-green1)', type: 'pen'},
         {model: 'purple1', class: 'ck5-pen-purple1', title: 'purple1', color: 'var(--ck-highlight-pen-purple1)', type: 'pen'},
         {model: 'cyan1', class: 'ck5-pen-cyan1', title: 'cyan1', color: 'var(--ck-highlight-pen-cyan1)', type: 'pen'},
         {model: 'orange1', class: 'ck5-pen-orange1', title: 'orange1', color: 'var(--ck-highlight-pen-orange1)', type: 'pen'},
         {model: 'yellow1', class: 'ck5-pen-yellow1', title: 'yellow1', color: 'var(--ck-highlight-pen-yellow1)', type: 'pen'},
         {model: 'black2', class: 'ck5-pen-black2', title: 'black2', color: 'var(--ck-highlight-pen-black2)', type: 'pen'},
         {model: 'white2', class: 'ck5-pen-white2', title: 'White2', color: 'var(--ck-highlight-pen-white2)', type: 'pen'},
         {model: 'beige2', class: 'ck5-pen-beige2', title: 'beige2', color: 'var(--ck-highlight-pen-beige2)', type: 'pen'},
         {model: 'blue2', class: 'ck5-pen-blue2', title: 'blue2', color: 'var(--ck-highlight-pen-blue2)', type: 'pen'},
         {model: 'red2', class: 'ck5-pen-red2', title: 'red2', color: 'var(--ck-highlight-pen-red2)', type: 'pen'},
         {model: 'green2', class: 'ck5-pen-green2', title: 'green2', color: 'var(--ck-highlight-pen-green2)', type: 'pen'},
         {model: 'purple2', class: 'ck5-pen-purple2', title: 'purple2', color: 'var(--ck-highlight-pen-purple2)', type: 'pen'},
         {model: 'cyan2', class: 'ck5-pen-cyan2', title: 'cyan2', color: 'var(--ck-highlight-pen-cyan2)', type: 'pen'},
         {model: 'orange2', class: 'ck5-pen-orange2', title: 'orange2', color: 'var(--ck-highlight-pen-orange2)', type: 'pen'},
         {model: 'yellow2', class: 'ck5-pen-yellow2', title: 'yellow2', color: 'var(--ck-highlight-pen-yellow2)', type: 'pen'},
         {model: 'black3', class: 'ck5-pen-black3', title: 'black3', color: 'var(--ck-highlight-pen-black3)', type: 'pen'},
         {model: 'white3', class: 'ck5-pen-white3', title: 'White3', color: 'var(--ck-highlight-pen-white3)', type: 'pen'},
         {model: 'beige3', class: 'ck5-pen-beige3', title: 'beige3', color: 'var(--ck-highlight-pen-beige3)', type: 'pen'},
         {model: 'blue3', class: 'ck5-pen-blue3', title: 'blue3', color: 'var(--ck-highlight-pen-blue3)', type: 'pen'},
         {model: 'red3', class: 'ck5-pen-red3', title: 'red3', color: 'var(--ck-highlight-pen-red3)', type: 'pen'},
         {model: 'green3', class: 'ck5-pen-green3', title: 'green3', color: 'var(--ck-highlight-pen-green3)', type: 'pen'},
         {model: 'purple3', class: 'ck5-pen-purple3', title: 'purple3', color: 'var(--ck-highlight-pen-purple3)', type: 'pen'},
         {model: 'cyan3', class: 'ck5-pen-cyan3', title: 'cyan3', color: 'var(--ck-highlight-pen-cyan3)', type: 'pen'},
         {model: 'orange3', class: 'ck5-pen-orange3', title: 'orange3', color: 'var(--ck-highlight-pen-orange3)', type: 'pen'},
         {model: 'yellow3', class: 'ck5-pen-yellow3', title: 'yellow3', color: 'var(--ck-highlight-pen-yellow3)', type: 'pen'},
         {model: 'black4', class: 'ck5-pen-black4', title: 'black4', color: 'var(--ck-highlight-pen-black4)', type: 'pen'},
         {model: 'white4', class: 'ck5-pen-white4', title: 'White4', color: 'var(--ck-highlight-pen-white4)', type: 'pen'},
         {model: 'beige4', class: 'ck5-pen-beige4', title: 'beige4', color: 'var(--ck-highlight-pen-beige4)', type: 'pen'},
         {model: 'blue4', class: 'ck5-pen-blue4', title: 'blue4', color: 'var(--ck-highlight-pen-blue4)', type: 'pen'},
         {model: 'red4', class: 'ck5-pen-red4', title: 'red4', color: 'var(--ck-highlight-pen-red4)', type: 'pen'},
         {model: 'green4', class: 'ck5-pen-green4', title: 'green4', color: 'var(--ck-highlight-pen-green4)', type: 'pen'},
         {model: 'purple4', class: 'ck5-pen-purple4', title: 'purple4', color: 'var(--ck-highlight-pen-purple4)', type: 'pen'},
         {model: 'cyan4', class: 'ck5-pen-cyan4', title: 'cyan4', color: 'var(--ck-highlight-pen-cyan4)', type: 'pen'},
         {model: 'orange4', class: 'ck5-pen-orange4', title: 'orange4', color: 'var(--ck-highlight-pen-orange4)', type: 'pen'},
         {model: 'yellow4', class: 'ck5-pen-yellow4', title: 'yellow4', color: 'var(--ck-highlight-pen-yellow4)', type: 'pen'},
         {model: 'black5', class: 'ck5-pen-black5', title: 'black5', color: 'var(--ck-highlight-pen-black5)', type: 'pen'},
         {model: 'white5', class: 'ck5-pen-white5', title: 'White5', color: 'var(--ck-highlight-pen-white5)', type: 'pen'},
         {model: 'beige5', class: 'ck5-pen-beige5', title: 'beige5', color: 'var(--ck-highlight-pen-beige5)', type: 'pen'},
         {model: 'blue5', class: 'ck5-pen-blue5', title: 'blue5', color: 'var(--ck-highlight-pen-blue5)', type: 'pen'},
         {model: 'red5', class: 'ck5-pen-red5', title: 'red5', color: 'var(--ck-highlight-pen-red5)', type: 'pen'},
         {model: 'green5', class: 'ck5-pen-green5', title: 'green5', color: 'var(--ck-highlight-pen-green5)', type: 'pen'},
         {model: 'purple5', class: 'ck5-pen-purple5', title: 'purple5', color: 'var(--ck-highlight-pen-purple5)', type: 'pen'},
         {model: 'cyan5', class: 'ck5-pen-cyan5', title: 'cyan5', color: 'var(--ck-highlight-pen-cyan5)', type: 'pen'},
         {model: 'orange5', class: 'ck5-pen-orange5', title: 'orange5', color: 'var(--ck-highlight-pen-orange5)', type: 'pen'},
         {model: 'yellow5', class: 'ck5-pen-yellow5', title: 'yellow5', color: 'var(--ck-highlight-pen-yellow5)', type: 'pen'},
         {model: 'black6', class: 'ck5-pen-black6', title: 'black6', color: 'var(--ck-highlight-pen-black6)', type: 'pen'},
         {model: 'white6', class: 'ck5-pen-white6', title: 'White6', color: 'var(--ck-highlight-pen-white6)', type: 'pen'},
         {model: 'beige6', class: 'ck5-pen-beige6', title: 'beige6', color: 'var(--ck-highlight-pen-beige6)', type: 'pen'},
         {model: 'blue6', class: 'ck5-pen-blue6', title: 'blue6', color: 'var(--ck-highlight-pen-blue6)', type: 'pen'},
         {model: 'red6', class: 'ck5-pen-red6', title: 'red6', color: 'var(--ck-highlight-pen-red6)', type: 'pen'},
         {model: 'green6', class: 'ck5-pen-green6', title: 'green6', color: 'var(--ck-highlight-pen-green6)', type: 'pen'},
         {model: 'purple6', class: 'ck5-pen-purple6', title: 'purple6', color: 'var(--ck-highlight-pen-purple6)', type: 'pen'},
         {model: 'cyan6', class: 'ck5-pen-cyan6', title: 'cyan6', color: 'var(--ck-highlight-pen-cyan6)', type: 'pen'},
         {model: 'orange6', class: 'ck5-pen-orange6', title: 'orange6', color: 'var(--ck-highlight-pen-orange6)', type: 'pen'},
         {model: 'yellow6', class: 'ck5-pen-yellow6', title: 'yellow6', color: 'var(--ck-highlight-pen-yellow6)', type: 'pen'},				
			]
		} );
	}

	/**
	 * @inheritDoc
	 */
	init() {
		const editor = this.editor;

		// Allow highlight attribute on text nodes.
		editor.model.schema.extend( '$text', { allowAttributes: 'highlight' } );

		const options = editor.config.get( 'highlight.options' );

		// Set-up the two-way conversion.
		editor.conversion.attributeToElement( _buildDefinition( options ) );

		editor.commands.add( 'highlight', new HighlightCommand( editor ) );
	}
}

// Converts the options array to a converter definition.
//
// @param {Array.<module:highlight/highlight~HighlightOption>} options An array with configured options.
// @returns {module:engine/conversion/conversion~ConverterDefinition}
function _buildDefinition( options ) {
	const definition = {
		model: {
			key: 'highlight',
			values: []
		},
		view: {}
	};

	for ( const option of options ) {
		definition.model.values.push( option.model );
		definition.view[ option.model ] = {
			name: 'span',
			classes: option.class
		};
	}

	return definition;
}
